/*******************************************************************************
* File Name: SPIM32_1_INT.c
* Version 2.40
*
* Description:
*  This file provides all Interrupt Service Routine (ISR) for the SPI Master
*  component.
*
* Note:
*  None.
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SPIM32_1_PVT.h"

/* User code required at start of ISR */
/* `#START SPIM32_1_ISR_START_DEF` */

/* `#END` */


/*******************************************************************************
* Function Name: SPIM32_1_TX_ISR
********************************************************************************
*
* Summary:
*  Interrupt Service Routine for TX portion of the SPI Master.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  SPIM32_1_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer.
*  SPIM32_1_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer, modified when exist data to
*  sending and FIFO Not Full.
*  SPIM32_1_txBuffer[SPIM32_1_TX_BUFFER_SIZE] - used to store
*  data to sending.
*  All described above Global variables are used when Software Buffer is used.
*
*******************************************************************************/
CY_ISR(SPIM32_1_TX_ISR)
{
    #if(SPIM32_1_TX_SOFTWARE_BUF_ENABLED)
        uint8 tmpStatus;
    #endif /* (SPIM32_1_TX_SOFTWARE_BUF_ENABLED) */

    /* User code required at start of ISR */
    /* `#START SPIM32_1_TX_ISR_START` */

    /* `#END` */

    #if(SPIM32_1_TX_SOFTWARE_BUF_ENABLED)
        /* Check if TX data buffer is not empty and there is space in TX FIFO */
        while(SPIM32_1_txBufferRead != SPIM32_1_txBufferWrite)
        {
            tmpStatus = SPIM32_1_GET_STATUS_TX(SPIM32_1_swStatusTx);
            SPIM32_1_swStatusTx = tmpStatus;

            if(0u != (SPIM32_1_swStatusTx & SPIM32_1_STS_TX_FIFO_NOT_FULL))
            {
                if(0u == SPIM32_1_txBufferFull)
                {
                   SPIM32_1_txBufferRead++;

                    if(SPIM32_1_txBufferRead >= SPIM32_1_TX_BUFFER_SIZE)
                    {
                        SPIM32_1_txBufferRead = 0u;
                    }
                }
                else
                {
                    SPIM32_1_txBufferFull = 0u;
                }

                /* Move data from the Buffer to the FIFO */
                CY_SET_REG8(SPIM32_1_TXDATA_PTR,
                    SPIM32_1_txBuffer[SPIM32_1_txBufferRead]);
            }
            else
            {
                break;
            }
        }

        if(SPIM32_1_txBufferRead == SPIM32_1_txBufferWrite)
        {
            /* TX Buffer is EMPTY: disable interrupt on TX NOT FULL */
            SPIM32_1_TX_STATUS_MASK_REG &= ((uint8) ~SPIM32_1_STS_TX_FIFO_NOT_FULL);
        }

    #endif /* (SPIM32_1_TX_SOFTWARE_BUF_ENABLED) */

    /* User code required at end of ISR (Optional) */
    /* `#START SPIM32_1_TX_ISR_END` */

    /* `#END` */
}


/*******************************************************************************
* Function Name: SPIM32_1_RX_ISR
********************************************************************************
*
* Summary:
*  Interrupt Service Routine for RX portion of the SPI Master.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  SPIM32_1_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer modified when FIFO contains
*  new data.
*  SPIM32_1_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer, modified when overflow occurred.
*  SPIM32_1_rxBuffer[SPIM32_1_RX_BUFFER_SIZE] - used to store
*  received data, modified when FIFO contains new data.
*  All described above Global variables are used when Software Buffer is used.
*
*******************************************************************************/
CY_ISR(SPIM32_1_RX_ISR)
{
    #if(SPIM32_1_RX_SOFTWARE_BUF_ENABLED)
        uint8 tmpStatus;
        uint8 rxData;
    #endif /* (SPIM32_1_RX_SOFTWARE_BUF_ENABLED) */

    /* User code required at start of ISR */
    /* `#START SPIM32_1_RX_ISR_START` */

    /* `#END` */

    #if(SPIM32_1_RX_SOFTWARE_BUF_ENABLED)

        tmpStatus = SPIM32_1_GET_STATUS_RX(SPIM32_1_swStatusRx);
        SPIM32_1_swStatusRx = tmpStatus;

        /* Check if RX data FIFO has some data to be moved into the RX Buffer */
        while(0u != (SPIM32_1_swStatusRx & SPIM32_1_STS_RX_FIFO_NOT_EMPTY))
        {
            rxData = CY_GET_REG8(SPIM32_1_RXDATA_PTR);

            /* Set next pointer. */
            SPIM32_1_rxBufferWrite++;
            if(SPIM32_1_rxBufferWrite >= SPIM32_1_RX_BUFFER_SIZE)
            {
                SPIM32_1_rxBufferWrite = 0u;
            }

            if(SPIM32_1_rxBufferWrite == SPIM32_1_rxBufferRead)
            {
                SPIM32_1_rxBufferRead++;
                if(SPIM32_1_rxBufferRead >= SPIM32_1_RX_BUFFER_SIZE)
                {
                    SPIM32_1_rxBufferRead = 0u;
                }

                SPIM32_1_rxBufferFull = 1u;
            }

            /* Move data from the FIFO to the Buffer */
            SPIM32_1_rxBuffer[SPIM32_1_rxBufferWrite] = rxData;

            tmpStatus = SPIM32_1_GET_STATUS_RX(SPIM32_1_swStatusRx);
            SPIM32_1_swStatusRx = tmpStatus;
        }

    #endif /* (SPIM32_1_RX_SOFTWARE_BUF_ENABLED) */

    /* User code required at end of ISR (Optional) */
    /* `#START SPIM32_1_RX_ISR_END` */

    /* `#END` */
}

/* [] END OF FILE */
