/*******************************************************************************
* File Name: SPIM32_1_PM.c
* Version 2.40
*
* Description:
*  This file contains the setup, control and status commands to support
*  component operations in low power mode.
*
* Note:
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SPIM32_1_PVT.h"

static SPIM32_1_BACKUP_STRUCT SPIM32_1_backup =
{
    SPIM32_1_DISABLED,
    SPIM32_1_BITCTR_INIT,
    #if(CY_UDB_V0)
        SPIM32_1_TX_INIT_INTERRUPTS_MASK,
        SPIM32_1_RX_INIT_INTERRUPTS_MASK
    #endif /* CY_UDB_V0 */
};


/*******************************************************************************
* Function Name: SPIM32_1_SaveConfig
********************************************************************************
*
* Summary:
*  Saves SPIM configuration.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  SPIM32_1_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void SPIM32_1_SaveConfig(void) 
{
    /* Store Status Mask registers */
    #if(CY_UDB_V0)
       SPIM32_1_backup.cntrPeriod      = SPIM32_1_COUNTER_PERIOD_REG;
       SPIM32_1_backup.saveSrTxIntMask = SPIM32_1_TX_STATUS_MASK_REG;
       SPIM32_1_backup.saveSrRxIntMask = SPIM32_1_RX_STATUS_MASK_REG;
    #endif /* (CY_UDB_V0) */
}


/*******************************************************************************
* Function Name: SPIM32_1_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores SPIM configuration.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  SPIM32_1_backup - used when non-retention registers are restored.
*
* Side Effects:
*  If this API is called without first calling SaveConfig then in the following
*  registers will be default values from Customizer:
*  SPIM32_1_STATUS_MASK_REG and SPIM32_1_COUNTER_PERIOD_REG.
*
*******************************************************************************/
void SPIM32_1_RestoreConfig(void) 
{
    /* Restore the data, saved by SaveConfig() function */
    #if(CY_UDB_V0)
        SPIM32_1_COUNTER_PERIOD_REG = SPIM32_1_backup.cntrPeriod;
        SPIM32_1_TX_STATUS_MASK_REG = ((uint8) SPIM32_1_backup.saveSrTxIntMask);
        SPIM32_1_RX_STATUS_MASK_REG = ((uint8) SPIM32_1_backup.saveSrRxIntMask);
    #endif /* (CY_UDB_V0) */
}


/*******************************************************************************
* Function Name: SPIM32_1_Sleep
********************************************************************************
*
* Summary:
*  Prepare SPIM Component goes to sleep.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  SPIM32_1_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void SPIM32_1_Sleep(void) 
{
    /* Save components enable state */
    SPIM32_1_backup.enableState = ((uint8) SPIM32_1_IS_ENABLED);

    SPIM32_1_Stop();
    SPIM32_1_SaveConfig();
}


/*******************************************************************************
* Function Name: SPIM32_1_Wakeup
********************************************************************************
*
* Summary:
*  Prepare SPIM Component to wake up.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  SPIM32_1_backup - used when non-retention registers are restored.
*  SPIM32_1_txBufferWrite - modified every function call - resets to
*  zero.
*  SPIM32_1_txBufferRead - modified every function call - resets to
*  zero.
*  SPIM32_1_rxBufferWrite - modified every function call - resets to
*  zero.
*  SPIM32_1_rxBufferRead - modified every function call - resets to
*  zero.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void SPIM32_1_Wakeup(void) 
{
    SPIM32_1_RestoreConfig();

    #if(SPIM32_1_RX_SOFTWARE_BUF_ENABLED)
        SPIM32_1_rxBufferFull  = 0u;
        SPIM32_1_rxBufferRead  = 0u;
        SPIM32_1_rxBufferWrite = 0u;
    #endif /* (SPIM32_1_RX_SOFTWARE_BUF_ENABLED) */

    #if(SPIM32_1_TX_SOFTWARE_BUF_ENABLED)
        SPIM32_1_txBufferFull  = 0u;
        SPIM32_1_txBufferRead  = 0u;
        SPIM32_1_txBufferWrite = 0u;
    #endif /* (SPIM32_1_TX_SOFTWARE_BUF_ENABLED) */

    /* Clear any data from the RX and TX FIFO */
    SPIM32_1_ClearFIFO();

    /* Restore components block enable state */
    if(0u != SPIM32_1_backup.enableState)
    {
        SPIM32_1_Enable();
    }
}


/* [] END OF FILE */
